/*****************************************************************************/
/*                                                                           */
/* Author: Michael C. Nechyba                                                */
/* Last modified: 5/22/96                                                    */
/*                                                                           */
/*****************************************************************************/

/* 
   Q represents a diagonal positive semidefinit matrix which is used
   to achieve a weighted distortion measure.
*/

typedef struct lbg_struct
{
  double **data;
  double **centroid;
  int *class;
  double *q;
  int n;
  int length;
  int M;
} LBG_t;

#define L(x) local->x

extern void LBG_one(LBG_t *local, double epsilon, double delta_dist,
		    int split);
extern int *int_copy_vector(int *old_vector, int *new_vector, int n);
extern void initialize_centroid(LBG_t *local);
extern void calculate_centroid(LBG_t *local, double epsilon);
extern int max_class_distortion(LBG_t *local, int class);
extern double *class_distortion(LBG_t *local, double *dist);
extern double total_distortion(LBG_t *local);
extern double distortion(double *vec1, double *vec2, int length);
extern void classify(LBG_t *local);
extern int classify_one(double *data_vec, int length, 
			double **centroid, int M);
extern int split_centroid(double **centroid, int length, int M,
			  double epsilon);
extern void add_epsilon_vec(double *vec1, double *vec2, int n, double epsilon);
extern void k_means(LBG_t *local);

extern LBG_t *read_centroid_file(LBG_t *local, char *codebook_file, 
				 int vec_length);
extern LBG_t *adjust_vectors_by_q(LBG_t *local, char *q_file);

extern void reassign_centroid(LBG_t *local, int *empty);

extern LBG_t *pick_columns(LBG_t *local, char *in_file, char *filter_file);
extern int split_centroid_rand(double **centroid, int length, int M, 
			       double epsilon);
extern void add_epsilon_vec_rand(double *vec1, double *vec2, int n, double *p);
