/*****************************************************************************/
/*                                                                           */
/* Description: VQ a file of vectors                                         */
/* File: "vq_main.c"                                                         */
/*                                                                           */
/* Author: Michael C. Nechyba                                                */
/* Last modified: 7/7/97                                                     */
/*                                                                           */
/*****************************************************************************/

#include <stdio.h>
#include <math.h>
#include "matrix.h"
#include "file_io.h"
#include "gen_math.h"
#include "codebook.h"
#include "arg.h"

#define LEVELS_MAX 4096

/**************************** Main Program ***********************************/

int main(int argc, char **argv)
{
  int i, sw;
  FILE *fp;
  char *in_file = NULL, *out_file = NULL, *codebook_file = NULL;
  char *q_file = NULL, *filter_file = NULL;
  LBG_t *local;
  int length = 16, header = 0, dist_prnt = 0, n_vec[LEVELS_MAX];
  double dis, dist_vec[LEVELS_MAX];

  arg_parse(argc, argv,
	    "", "Usage:",
	    "-i %S", &in_file,  "set input file",
	    "-o %S", &out_file, "set output file",
	    "-c %S", &codebook_file, "set codebook file",
	    "-f [%S]", &filter_file, "set filter file [none]",
	    "-header", ARG_FLAG(&header), "include header in output file",
	    "-q [%S]", &q_file, "set Q file [none]",
	    "-l [%d]", &length, "set vector_length [%d]", length,
	    "-distortion", ARG_FLAG(&dist_prnt), 
	    "print distortion for individual codes",

	    NULL);

  if(in_file == NULL || out_file == NULL || codebook_file == NULL) {
    fprintf(stderr, "No input and/or output file specified.\n");
    exit(-1);
  }

  local = (LBG_t *) malloc(sizeof(LBG_t));
  is_null((int *) local);

  L(n) = count_lines(in_file);

  L(length) = length;

  pick_columns(local, in_file, filter_file);

  adjust_vectors_by_q(local, q_file);

  L(M) = count_lines(codebook_file);

  L(centroid) = allocate_matrix(L(data), L(M), L(length));
  L(class) = int_allocate_vector(L(class), L(n));
  L(class) = int_zero_vector(L(class), L(n));

  fp = fopen(codebook_file, "r");
  read_matrix(fp, L(centroid), L(M), L(length));
  fclose(fp);

  classify(local);

  dis = total_distortion(local);
  fprintf(stdout, 
	  "Distortion: (Total = %lf ), (normalized = %.10f )\n", 
	  sqrt(dis), sqrt(dis/((double) (L(length)*L(n)))));

  fp = fopen(out_file, "w");
  if(header) {
    fprintf(fp, "1 %d\n", L(M));
    fprintf(fp, "%d %s\n", L(n), out_file);
  }

  if(dist_prnt) {
    for(i = 0; i < L(M); i++) {
      dist_vec[i] = 0.;
      n_vec[i] = 0;
    }

    for(i = 0; i < L(n); i++) {
      dist_vec[L(class[i])] += 
	distortion(L(data[i]), L(centroid[L(class[i])]), L(length));
      n_vec[L(class[i])]++;
    }

    for(i = 0; i < L(M); i++) {
      if(n_vec[i] <= 1) {
	fprintf(stdout, "%d : %.15f / ( %d - 1 ) = %.15f ( 0. )\n", 
		i, dist_vec[i], n_vec[i], dist_vec[i]);
      } else {
	fprintf(stdout, "%d : %.15f / ( %d - 1 ) = %.15f ( %.15f )\n", 
		i, dist_vec[i], n_vec[i], 
		dist_vec[i]/((double)(n_vec[i] - 1.)),
		log10(dist_vec[i]/((double)(n_vec[i] - 1.))));
      }
    }
  }
   
  for(i = 0; i < L(n); i++)
    fprintf(fp, "%d\n", L(class[i]));
  fclose(fp);
}

/*****************************************************************************/
